# -*- coding: UTF-8 -*-
"""
Name: opendir.py
Porpose: open file browser on given pathname
Compatibility: Python3 (Unix, Windows)
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2022 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Feb.22.2022
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import subprocess
import os


def browse(ostype, pathname):
    """
    open file in a specific location (OS independent)

    """
    if ostype == 'Windows':
        try:
            os.startfile(os.path.realpath(pathname))

        except FileNotFoundError as pathnotfound:
            return f'{pathnotfound}'

        except Exception as anyerr:
            return f'{anyerr}'

        return None

    if ostype == 'Darwin':
        cmd = ['open', pathname]

    else:  # xdg-open *should* be supported by recent Gnome, KDE, Xfce
        cmd = ['xdg-open', pathname]

    try:
        proc = subprocess.run(cmd, check=True, shell=False)
    except FileNotFoundError as err:
        return err

    if proc.returncode:
        return "EXIT: {proc.returncode}\nERROR: {proc.stderr}"

    return None



# -*- coding: UTF-8 -*-
"""
Name: one_pass.py
Porpose: FFmpeg long processing task for one pass processing
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2022 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: March.10.2022
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from threading import Thread
import time
import itertools
import subprocess
import platform
import wx
from pubsub import pub
from videomass.vdms_utils.utils import Popen
from videomass.vdms_io.make_filelog import logwrite
if not platform.system() == 'Windows':
    import shlex


class OnePass(Thread):
    """
    This class represents a separate thread for running processes,
    which need to read the stdout/stderr in real time.

    NOTE capturing output in real-time (Windows, Unix):

    https://stackoverflow.com/questions/1388753/how-to-get-output-
    from-subprocess-popen-proc-stdout-readline-blocks-no-dat?rq=1

    """
    get = wx.GetApp()  # get videomass wx.App attribute
    appdata = get.appset
    SUFFIX = appdata['filesuffix']
    NOT_EXIST_MSG = _("Is 'ffmpeg' installed on your system?")
    # ---------------------------------------------------------------

    def __init__(self, varargs, duration, logname, timeseq):
        """
        Some attribute can be empty, this depend from conversion type.
        If the format/container is not changed on a conversion, the
        'extoutput' attribute will have an empty value.
        The 'volume' attribute may also have an empty value, but it will
        no affect as well.
        """
        self.stop_work_thread = False  # process terminate
        self.filelist = varargs[1]  # list of files (items)
        self.command = varargs[4]  # comand set on single pass
        self.outputdir = varargs[3]  # output path
        self.extoutput = varargs[2]  # format (extension)
        self.duration = duration  # duration list
        self.volume = varargs[7]  # (lista norm.)se non richiesto rimane None
        self.count = 0  # count first for loop
        self.countmax = len(varargs[1])  # length file list
        self.logname = logname  # title name of file log
        self.time_seq = timeseq  # a time segment

        Thread.__init__(self)

        self.start()  # start the thread

    def run(self):
        """
        Thread started.
        """
        filedone = []
        for (files,
             fold,
             volume,
             duration) in itertools.zip_longest(self.filelist,
                                                self.outputdir,
                                                self.volume,
                                                self.duration,
                                                fillvalue='',
                                                ):
            basename = os.path.basename(files)  # nome file senza path
            fname = os.path.splitext(basename)[0]  # nome senza estensione
            source_ext = os.path.splitext(basename)[1].split('.')[1]  # ext
            outext = source_ext if not self.extoutput else self.extoutput
            outfile = os.path.join(fold, f'{fname}{OnePass.SUFFIX}.{outext}')

            cmd = (f'"{OnePass.appdata["ffmpeg_cmd"]}" {self.time_seq} '
                   f'{OnePass.appdata["ffmpegloglev"]} '
                   f'{OnePass.appdata["ffmpeg+params"]} -i '
                   f'"{files}" {self.command} {volume} '
                   f'{OnePass.appdata["ffthreads"]} -y "{outfile}"')

            self.count += 1
            count = f'File {self.count}/{self.countmax}'
            com = (f'{count}\nSource: "{files}"\nDestination: "{outfile}"'
                   f'\n\n[COMMAND]:\n{cmd}')

            wx.CallAfter(pub.sendMessage,
                         "COUNT_EVT",
                         count=count,
                         fsource=f'Source:  "{files}"',
                         destination=f'Destination:  "{outfile}"',
                         duration=duration,
                         end='',
                         )
            logwrite(com, '', self.logname)  # write n/n + command only

            if not platform.system() == 'Windows':
                cmd = shlex.split(cmd)

            try:
                with Popen(cmd,
                           stderr=subprocess.PIPE,
                           bufsize=1,
                           universal_newlines=True,
                           ) as proc:
                    for line in proc.stderr:
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output=line,
                                     duration=duration,
                                     status=0,
                                     )
                        if self.stop_work_thread:
                            proc.terminate()
                            break  # break second 'for' loop

                    if proc.wait():  # error
                        wx.CallAfter(pub.sendMessage,
                                     "UPDATE_EVT",
                                     output='',
                                     duration=duration,
                                     status=proc.wait(),
                                     )
                        logwrite('',
                                 f"Exit status: {proc.wait()}",
                                 self.logname,
                                 )  # append exit error number
                    else:  # ok
                        filedone.append(files)
                        wx.CallAfter(pub.sendMessage,
                                     "COUNT_EVT",
                                     count='',
                                     fsource='',
                                     destination='',
                                     duration=duration,
                                     end='ok'
                                     )
            except (OSError, FileNotFoundError) as err:
                excepterr = f"{err}\n  {OnePass.NOT_EXIST_MSG}"
                wx.CallAfter(pub.sendMessage,
                             "COUNT_EVT",
                             count=excepterr,
                             fsource='',
                             destination='',
                             duration=0,
                             end='error',
                             )
                break

            if self.stop_work_thread:
                proc.terminate()
                break  # break second 'for' loop

        time.sleep(.5)
        wx.CallAfter(pub.sendMessage, "END_EVT", msg=filedone)
    # --------------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True



# -*- coding: UTF-8 -*-
"""
File Name: checkup.py
Porpose: input/output file check
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2022 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.18.2022
Code checker:
    flake8: --ignore F821
    pylint: --ignore E0602, E1101
########################################################

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import wx


def check_inout(exclude,
                file_sources,
                outputdir
                ):
    """
    check for overwriting and file/dir existence.

    return following values:

    file_sources: a file list filtered by checking.
    outputdir: contains output paths as many as the file sources
    lenghmax: lengh list useful for count the loop index on the batch process.

    if already exists or file does not exist or output folder
    does not exist, return
        (False, None, None, None, None) .

    """
    if exclude:
        if wx.MessageBox(_('Already exist: \n\n{}\n\n'
                           'Do you want to overwrite? ').format(
                         '\n'.join(exclude)),
                         _('Please Confirm'),
                         wx.ICON_QUESTION | wx.YES_NO, None) == wx.NO:
            return None

    # --------------- CHECK EXISTING FILES AND DIRECTORIES:
    for fln in file_sources:
        if not os.path.isfile(os.path.abspath(fln)):
            wx.MessageBox(_('File does not exist:\n\n"{}"\n').format(fln),
                          "Videomass", wx.ICON_ERROR
                          )
            return None

    for drn in outputdir:
        if not os.path.isdir(os.path.abspath(drn)):
            wx.MessageBox(_('Output folder does not exist:\n\n"{}"\n').format(
                          drn), 'Videomass', wx.ICON_ERROR
                          )
            return None

    return (file_sources, outputdir, len(file_sources))


def check_files(file_sources,
                dir_destin,
                same_destin,
                suffix,
                extout
                ):
    """
    Creates the data structures relating to the files to
    be processed and to be excluded if they exist in the
    same path.

    if no file or one of the conditions returned by the
    check_inout function, return
        (False, None, None, None, None)

    return (file_sources, outputdir, len(file_sources)) otherwise

    """
    if not file_sources:
        return None

    exclude = []  # already exist file names list
    outputdir = []  # output path names list
    # base_name = []  # represents the complete final basenames

    # --------------- CHECK OVERWRITING:
    for path in file_sources:
        dirname = os.path.dirname(path)
        basename = os.path.basename(path)
        filename = os.path.splitext(basename)  # [name, ext]

        if not extout:  # uses more extensions (copy formats)
            if same_destin:
                pathname = f'{dirname}/{filename[0]}{suffix}{filename[1]}'
                outputdir.append(dirname)
                # base_name.append(os.path.basename(pathname))
            else:
                pathname = f'{dir_destin}/{basename}'
                outputdir.append(dir_destin)
                # base_name.append(basename)

            if os.path.exists(pathname):
                exclude.append(pathname)

        else:  # uses one extension for all output
            if same_destin:
                pathname = f'{dirname}/{filename[0]}{suffix}.{extout}'
                outputdir.append(dirname)
            else:
                pathname = f'{dir_destin}/{filename[0]}.{extout}'
                outputdir.append(dir_destin)

            if os.path.exists(pathname):
                exclude.append(pathname)

    return check_inout(exclude, file_sources, outputdir)



# -*- coding: UTF-8 -*-
"""
Name: epilogue.py
Porpose: show dialog box before start process
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: May.15.2020
Code checker:
    - pylint: --ignore E0602, E1101, C0415, E0611, R0901,
    - pycodestyle

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""

import wx


class Formula(wx.Dialog):
    """
    Show a dialog box before run process.

    Example:
            settings = ("\nEXAMPLES:\n\nExample 1:\nExample 2:\n etc."
            param = ("type 1\ntype 2\ntype 3\n etc."
    """
    def __init__(self, parent, settings, param, title):

        get = wx.GetApp()  # get data from bootstrap
        colorscheme = get.appset['icontheme'][1]

        wx.Dialog.__init__(self, parent, -1, style=wx.DEFAULT_DIALOG_STYLE)
        sizbase = wx.BoxSizer(wx.VERTICAL)
        panel = wx.Panel(self, wx.ID_ANY, style=wx.TAB_TRAVERSAL)
        sizbase.Add(panel, 1, wx.ALL | wx.EXPAND, 5)
        label1 = wx.StaticText(panel, wx.ID_ANY, settings)
        label2 = wx.StaticText(panel, wx.ID_ANY, param)
        panel.SetBackgroundColour(colorscheme['BACKGRD'])
        label2.SetForegroundColour(colorscheme['TXT1'])
        label1.SetForegroundColour(colorscheme['TXT3'])
        grid_pan = wx.FlexGridSizer(1, 2, 0, 0)
        grid_pan.Add(label1, 1, wx.ALL | wx.ALIGN_CENTRE, 5)
        grid_pan.Add(label2, 1, wx.ALL | wx.ALIGN_CENTRE, 5)
        panel.SetSizer(grid_pan)

        self.button_1 = wx.Button(self, wx.ID_CANCEL, "")
        self.button_2 = wx.Button(self, wx.ID_OK, "")
        btngrid = wx.FlexGridSizer(1, 2, 0, 0)
        btngrid.Add(self.button_1, 0, wx.ALL, 5)
        btngrid.Add(self.button_2, 0, wx.ALL, 5)
        sizbase.Add(btngrid, flag=wx.ALL | wx.ALIGN_RIGHT | wx.RIGHT, border=0)

        self.SetTitle(title)
        self.SetSizer(sizbase)
        sizbase.Fit(self)
        self.Layout()
        # ----------------------Binders (EVT)--------------------#
        self.Bind(wx.EVT_BUTTON, self.on_cancel, self.button_1)
        self.Bind(wx.EVT_BUTTON, self.on_ok, self.button_2)

        # --------------  Event handler (callback)  --------------#

    def on_cancel(self, event):
        """
        exit from formula dialog
        """
        # self.Destroy()
        event.Skip()

    def on_ok(self, event):
        """
        get confirmation to proceed
        """
        # self.Destroy()
        event.Skip()



# -*- coding: UTF-8 -*-
"""
Name: settings_manager.py
Porpose: Read and write the configuration file
Compatibility: Python3
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2022 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: April.26.2022
Code checker: flake8, pylint .

 This file is part of Videomass.

    Videomass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Videomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import json


class ConfigManager:
    """
    It represents the setting of the user parameters
    of the program and the configuration file in its
    read and write aspects.

    Usage:

    write a new conf.json :
        >>> from settings_manager import ConfigManager
        >>> confmng = ConfigManager('fileconfpath.json')
        >>> confmng.write_options()

    read the current fileconf.json :
        >>> settings = confmng.read_options()

    example of modify data into current file conf.json:
        >>> settings['outputfile'] = '/home/user/MyVideos'
        >>> confmng.write_options(**settings)
    ------------------------------------------------------

    Options description:

    confversion (float):
        current version of this configuration file

    outputfile (str):
        output path used by ffmpeg for file saving

    outputfile_samedir (bool):
        if True save any ffmpeg files to same folder as source

    filesuffix (str):
        An optional suffix to assign to output files

    outputdownload (str):
        output path used by the downloader for file saving

    ffmpeg_cmd, ffplay_cmd, ffprobe_cmd, (str):
        Absolute or relative path name of the executable.
        if empty string ("") starts the wizard.

    ffmpeg_islocal, ffplay_islocal, ffprobe_islocal, (bool):
        With True the user enables the executable locally

    ffmpegloglev (str):
        -loglevel one of `error`, `warning`, `info`, `verbose`, `debug`

    ffthreads (str):
        Set the number of threads (from 0 to 32)

    ffplayloglev (str):
        -loglevel one of `quiet`, `fatal`, `error`, `warning`, `info`

    warnexiting (bool):
        with True displays a message dialog before exiting the app

    icontheme (str):
        Icon theme Name (see art folder)

    toolbarsize (int):
        Set toolbar icon size, one of 16, 24, 32, 64 default is 24 px.

    toolbarpos (int):
        Set toolbar positioning. 0 placed on top side;
        1 placed at the bottom side; 2 placed at the rigth side;
        3 is placed at the left side. default is 0 .

    toolbartext (bool):
        True, enables text alongside toolbar buttons. Default is False.

    panel_size (list):
        [Height, Width] current window dimension.

    clearcache (bool):
        with True deleting file cache on exit the app,
        default is False

    clearlogfiles (bool):
        with True deleting log file content on exit the app,
        default is False.

    downloader (bool, str):
        sets the downloader to use when application startup.
        one of `disabled`,` youtube_dl`, `yt_dlp` or False
        Where `disabled` means not load anything, `youtube_dl`
        means load/use youtube-dl on sturtup, `yt_dlp` means load/use
        yt_dl on sturtup, `false` means *not set at all* and the
        wizard dialog will be displayed.

    playlistsubfolder (bool):
        Auto-create subfolders when download the playlists,
        default value is True.

    move_file_to_trash (bool):
        if True, move input files to videomass trash folder or inside
        some other dir.
        default value is False

    trashfolder (str):
        An absolute or relative dirname of some trash folder.

    """
    VERSION = 4.2
    DEFAULT_OPTIONS = {
        "confversion": VERSION,
        "outputfile": f"{os.path.expanduser('~')}",
        "outputfile_samedir": False,
        "filesuffix": "",
        "outputdownload": f"{os.path.expanduser('~')}",
        "ffmpeg_cmd": "",
        "ffmpeg_islocal": False,
        "ffmpegloglev": "-loglevel warning",
        "ffthreads": "-threads 4",
        "ffplay_cmd": "",
        "ffplay_islocal": False,
        "ffplayloglev": "-loglevel error",
        "ffprobe_cmd": "",
        "ffprobe_islocal": False,
        "warnexiting": True,
        "icontheme": "Videomass-Colours",
        "toolbarsize": 24,
        "toolbarpos": 0,
        "toolbartext": False,
        "panel_size": [980, 640],
        "clearcache": False,
        "clearlogfiles": False,
        "downloader": False,
        "playlistsubfolder": True,
        "move_file_to_trash": False,
        "trashfolder": ""
        }

    def __init__(self, filename, relativepath=False):
        """
        Accepts an existing `filename` on the file system paths
        suffixed by `.json`. If `relativepath` is `True`, some
        paths on the `DEFAULT_OPTIONS` class attribute will be
        set as relative paths.
        """
        self.filename = filename
        if relativepath is True:
            appdir = os.getcwd()
            outputdir = os.path.relpath(os.path.join(appdir, 'My_Files'))
            ConfigManager.DEFAULT_OPTIONS['outputfile'] = outputdir
            ConfigManager.DEFAULT_OPTIONS['outputdownload'] = outputdir

    def write_options(self, **options):
        """
        Writes options to configuration file. If **options is
        given, writes the new changes to filename, writes the
        DEFAULT_OPTIONS otherwise.
        """
        if options:
            set_options = options
        else:
            set_options = ConfigManager.DEFAULT_OPTIONS

        with open(self.filename, "w", encoding='utf-8') as settings_file:

            json.dump(set_options,
                      settings_file,
                      indent=4,
                      separators=(",", ": ")
                      )

    def read_options(self):
        """
        Reads options from the current configuration file.
        Returns: current options, `None` otherwise.
        Raise: json.JSONDecodeError
        """
        with open(self.filename, 'r', encoding='utf-8') as settings_file:
            try:
                options = json.load(settings_file)
            except json.JSONDecodeError:
                return None

        return options



# -*- coding: UTF-8 -*-
"""
Name: ffplay_url.py
Porpose: playback online media streams with ffplay player using
         youtube_dl embedding on code.
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2022 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.02.2022
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
from threading import Thread
import wx
from pubsub import pub
from videomass.vdms_io import io_tools
if 'youtube_dl' in sys.modules:
    import youtube_dl
elif 'yt_dlp' in sys.modules:
    import yt_dlp


def msg_error(msg):
    """
    Receive error messages via wxCallafter
    """
    wx.MessageBox(f"{msg}", "Videomass", wx.ICON_ERROR)
# ------------------------------------------------------------------------#


def msg_warning(msg):
    """
    Receive info messages via wxCallafter
    """
    wx.MessageBox(f"{msg}", "Videomass", wx.ICON_INFORMATION)
# ------------------------------------------------------------------------#


class MyLogger():
    """
    Intercepts youtube-dl's output by setting a logger object;
    * Log messages to a logging.Logger instance.
    <https://github.com/ytdl-org/youtube-dl/tree/3e4cedf9e8cd31
    57df2457df7274d0c842421945#embedding-youtube-dl>
    """

    def __init__(self):
        """Initialize"""
        self.msg = None

    def debug(self, msg):
        """intercept msg"""
        # print('DEBUG: ', msg)
        if '[download]' in msg:  # ...in processing
            # print(msg)
            if 'Destination' in msg:
                # print(msg)
                pub.sendMessage("START_FFPLAY_EVT",
                                output=msg.split()[2]
                                )
            elif 'has already been downloaded' in msg:
                pub.sendMessage("START_FFPLAY_EVT",
                                output=msg.split()[1]
                                )
        self.msg = msg

    def warning(self, msg):
        """send warning messages to masg_"""
        wx.CallAfter(msg_warning, msg)

    def error(self, msg):
        """send message errors"""
        wx.CallAfter(msg_error, msg)
# -------------------------------------------------------------------------#


class DownloadStream(Thread):
    """
    Download media stream to the videomass cache directory with
    the same quality defined in the `quality` parameter.
    This Thread use embedding youtube_dl package.
    WARNING FIXME stop thead does not work since youtube_dl.YoutubeDL
                  library does not yet have an interface for stopping
                  unfinished downloads.

    The file name form stored on cache dir. is

            "title_" + "quality" + ".format"

    """
    get = wx.GetApp()  # get videomass wx.App attribute
    OS = get.appset['ostype']
    FFMPEG_URL = get.appset['ffmpeg_cmd']
    TMP = os.path.join(get.appset['cachedir'], 'tmp')
    APP = get.appset['app']
    DOWNLOADER = get.appset['downloader']

    def __init__(self, url, quality, ssl):
        """
        Accept a single url as a string.
        The quality parameter sets the quality of the stream
        in the form required by youtube-dl, e.g one of the
        following strings can be valid:
            worst, best, bestvideo+bestaudio, Format Code

        """
        self.stop_work_thread = False  # process terminate value
        self.url = url  # single url
        self.quality = quality  # output quality e.g. worst, best, Format code
        self.nocheckcertificate = ssl
        self.outputdir = DownloadStream.TMP  # pathname destination
        self.outtmpl = f'%(title)s_{self.quality}.%(ext)s'  # filename

        Thread.__init__(self)
    # --------------------------------------------------------------#

    def run(self):
        """
        This atipic method is called by start() method after the instance
        this class. see Streaming class below.
        """
        if self.stop_work_thread:
            return

        ydl_opts = {
            'format': self.quality,
            'outtmpl': f'{self.outputdir}/{self.outtmpl}',
            'restrictfilenames': True,
            'nopart': True,  # see --no-part by --help
            'ignoreerrors': True,
            'continue': True,
            'no_warnings': False,
            'noplaylist': True,
            'no_color': True,
            'nocheckcertificate': self.nocheckcertificate,
            'ffmpeg_location': f'{DownloadStream.FFMPEG_URL}',
            'logger': MyLogger(),
        }
        if DownloadStream.DOWNLOADER == 'yt_dlp':
            with yt_dlp.YoutubeDL(ydl_opts) as ydl:
                ydl.download([f"{self.url}"])

        elif DownloadStream.DOWNLOADER == 'youtube_dl':
            with youtube_dl.YoutubeDL(ydl_opts) as ydl:
                ydl.download([f"{self.url}"])
    # --------------------------------------------------------------#

    def stop(self):
        """
        Sets the stop work thread to terminate the process
        """
        self.stop_work_thread = True
# ------------------------------------------------------------------------#


class Streaming():
    """
    Handling Threads to download and playback media streams via
    youtube-dl library and ffmpeg executables.

    DOWNLOAD class variable makes the object's attributes available
    even outside of class, see `stop_download_listener()`
    FIXME put RICEIVER LISTENER function as methods of this class. For
          now does not work (???)

    """
    DOWNLOAD = None  # set instance thread
    TIMESTAMP = None
    AUTOEXIT = None
    # ---------------------------------------------------------------#

    def __init__(self, timestamp, autoexit, ssl, url=None, quality=None):
        """
        - Topic "START_FFPLAY_EVT" subscribes the start playing
          running ffplay at a certain time.
        - Topic "STOP_DOWNLOAD_EVT" subscribes a stop download listener
          which call the stop() method of `DownloadStream` class to
          stop the download when ffplay has finished or been closed by
          the user.
        """
        pub.subscribe(stop_download_listener, "STOP_DOWNLOAD_EVT")
        pub.subscribe(start_palying_listener, "START_FFPLAY_EVT")

        Streaming.DOWNLOAD = DownloadStream(url, quality, ssl)
        Streaming.TIMESTAMP = timestamp
        Streaming.AUTOEXIT = autoexit

        self.start_download()
    # ----------------------------------------------------------------#

    def start_download(self):
        """
        call DownloadStream(Thread) to run() method
        """
        Streaming.DOWNLOAD.start()

# --------- RECEIVER LISTENERS


def stop_download_listener(filename):
    """
    Receive message from ffplay_file.FilePlay class
    for handle interruption
    """
    Streaming.DOWNLOAD.stop()
    Streaming.DOWNLOAD.join()  # if join, wait end process


def start_palying_listener(output):
    """
    Riceive messages from MyLogger to start
    ffplay in at a given time.
    """
    io_tools.stream_play(output,
                         '',
                         Streaming.TIMESTAMP,
                         Streaming.AUTOEXIT
                         )



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
Name: rsvg2png
Porpose: Wrapper interface to perform batch conversion using the rsvg library
Compatibility: Python3
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2020-2021 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Oct.23.2020 *PEP8 compatible*

DESCRIPTION:
   Given a list of pathnames, recursively converts images found in SVG
   vector format to PNG raster format or to the formats specified in
   the --format argument.

       Assume that the `rsvg-convert` command is installed on the system:

       - Mac OS : brew install librsvg
       - Ubuntu : apt install librsvg2-bin
       - available even for Windows

#########################################################

This file is part of Videomass.

    Videomass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Videomass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Videomass.  If not, see <http://www.gnu.org/licenses/>.

"""
import platform
import subprocess
import os
import glob
import argparse
import sys


def svg2png(delete, paths, parameters,
            outputdir, recursive: bool = False):
    """
    Iterates on a generator given from `glob.iglob` method.
    glob treats filenames beginning with a dot (.) as special cases,
    so hidden filenames will not be considered. To convert hidden
    filenames use `fnmatch` module instead of `glob` module.

    """
    cmd = ['rsvg-convert']
    recurs = '**' if recursive else ''

    if parameters["-f"] == 'recording':
        outext = 'png'
    elif parameters["-f"]:
        outext = parameters["-f"]
    else:
        outext = 'png'

    for key, val in parameters.items():
        if val:
            cmd.append(key)
            cmd.append(str(val))

    for path in paths:
        for files in glob.iglob(os.path.join(path, recurs, '*.svg'),
                                recursive=recursive
                                ):
            out = outputdir if outputdir else os.path.dirname(files)
            basename = os.path.splitext(os.path.basename(files))
            saveto = os.path.join('%s' % out, '%s.%s' % (basename[0], outext))
            if platform.system() == 'Windows':
                command = ' '.join(cmd + [files, '-o', saveto])
            else:
                command = cmd + [files, '-o', saveto]
            try:
                proc = subprocess.run(command,
                                      capture_output=True,
                                      universal_newlines=True,
                                      check=True,
                                      )
            except FileNotFoundError:
                return "ERROR: 'rsvg-convert': Command not found"

            if proc.returncode:
                return "ARGS: %s\nERROR: %s" % (proc.args, proc.stderr)

            if delete:
                try:
                    os.remove(files)
                except OSError as err:
                    return "ERROR: %s" % err
    return None
    # -------------------------------------------------------------------#


def main():
    """
    Entry-point of the executable (users inputs parser, positional/optional
    arguments)

    """
    descrstr = ('Wrapper interface to perform batch conversion '
                'using the `rsvg` library. Given a list of '
                'pathnames, recursively converts images found in '
                'SVG vector format to PNG raster format or to '
                'the formats specified in the --format argument:'
                )
    parser = argparse.ArgumentParser(description=descrstr)
    parser.add_argument('-d', '--dpi',
                        required=False,
                        type=float,
                        help="pixels per inch; defaults to 90dpi",
                        )
    parser.add_argument('-W', '--width',
                        required=False,
                        type=int,
                        help="width; defaults to the SVG's width",
                        )
    parser.add_argument('-H', '--height',
                        required=False,
                        type=int,
                        help="height; defaults to the SVG's height",
                        )
    parser.add_argument('-f', '--format',
                        choices=['png', 'pdf', 'ps', 'eps',
                                 'svg', 'xml', 'recording'],
                        required=False,
                        type=str,
                        help="Save format; defaults to 'png'",
                        )
    parser.add_argument('-p', '--paths',
                        nargs='+',
                        metavar='..DIR ..DIR',
                        required=True,
                        help="Input dirname list separated by spaces",
                        )
    parser.add_argument('-r', '--remove',
                        action='store_true',
                        required=False,
                        help=("If successful, remove all SVG "
                              "source files when finished"),
                        )
    parser.add_argument('-R', '--recursive',
                        action='store_true',
                        required=False,
                        help=("Recursive use (please be careful "
                              "when used with "
                              "the `-r` argument)"),
                        )
    parser.add_argument('-o', '--output',
                        metavar='..DIR',
                        required=False,
                        help="Provide an output directory to save all files",
                        )
    args = parser.parse_args()

    for pth in args.paths:
        if not os.path.isdir(pth):
            raise NotADirectoryError("Invalid or inexistent pathname for "
                                     "inputdir '%s'" % p)

    if args.output:
        if not os.path.isdir(args.output):
            raise NotADirectoryError("Invalid or inexistent pathname for "
                                     "outputdir '%s'" % args.output)

    if args.format == 'svg' and not args.output:
        raise FileExistsError('Could not overwrite the SVG files themselves. '
                              'You must provide an output pathname using the '
                              '`-o` argument.')

    adds = {'-d': args.dpi, '-w': args.width,
            '-h': args.height, '-f': args.format}

    ret = svg2png(args.remove,
                  args.paths,
                  adds,
                  args.output,
                  args.recursive
                  )
    if ret:
        sys.exit(ret)
    else:
        print('Done! :-)')
        sys.exit()
    # ----------------------------------------------------------------#


if __name__ == '__main__':
    main()



# -*- coding: UTF-8 -*-
"""
Name: io_tools.py
Porpose: input/output redirection to processes (aka threads)
Compatibility: Python3, wxPython4 Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2022 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Mar.02.2022
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import requests
import wx
from videomass.vdms_threads.ffplay_file import FilePlay
from videomass.vdms_threads import (ffplay_url,
                                    generic_downloads,
                                    appimage_updater,
                                    )
from videomass.vdms_threads.ffprobe import ffprobe
from videomass.vdms_threads.volumedetect import VolumeDetectThread
from videomass.vdms_threads.check_bin import (ff_conf,
                                              ff_formats,
                                              ff_codecs,
                                              ff_topics,
                                              )
from videomass.vdms_threads.opendir import browse
from videomass.vdms_threads.ydl_extractinfo import YdlExtractInfo

from videomass.vdms_frames import (ffmpeg_conf,
                                   ffmpeg_formats,
                                   ffmpeg_codecs,
                                   )
from videomass.vdms_dialogs.widget_utils import PopupDialog
from videomass.vdms_io.make_filelog import make_log_template


def stream_info(title, filepath):
    """
    Show media information of the streams content.
    This function make a bit control of file existance.
    """
    get = wx.GetApp()
    try:
        with open(filepath, encoding='utf8'):
            miniframe = Mediainfo(title,
                                  filepath,
                                  get.appset['ffprobe_cmd'],
                                  get.appset['ostype'],
                                  )
            miniframe.Show()

    except IOError:
        wx.MessageBox(_("File does not exist or is invalid:  %s") % (
            filepath), "Videomass", wx.ICON_EXCLAMATION, None)
# -----------------------------------------------------------------------#


def stream_play(filepath, tseq, param, autoexit):
    """
    Call Thread for playback with ffplay
    """
    get = wx.GetApp()  # get data from bootstrap
    tseq = tseq if tseq != "-ss 00:00:00.000 -t 00:00:00.000" else ''
    try:
        with open(filepath, encoding='utf8'):
            FilePlay(filepath,
                     tseq,
                     param,
                     get.appset['logdir'],
                     get.appset['ffplay_cmd'],
                     get.appset['ffplayloglev'],
                     get.appset['ffplay+params'],
                     autoexit
                     )
            # thread.join() > attende fine thread, se no ritorna subito
            # error = thread.data
    except IOError:
        wx.MessageBox(_("File does not exist or is invalid:  %s") % (
            filepath), "Videomass", wx.ICON_EXCLAMATION, None)
        return
# -----------------------------------------------------------------------#


def url_play(url, quality, timestamp, autoexit, ssl):
    """
    directs to the corresponding thread for playing
    online media streams.

    NOTE I'm looking for a way to tell the youtube_dl thread to terminate
    (gracefully or not, whatever) when the user clicks the "stop" button.
    The command-line version of youtube_dl just catches KeyboardInterrupt
    and terminates, but I've found no documentation on how to do this
    cleanly (if there's a way at all) when using the embedded module.
    So I'm still forced to use youtube-dl with the command-line and
    subprocess module.
    See https://github.com/ytdl-org/youtube-dl/issues/16175

    """
    ffplay_url.Streaming(timestamp, autoexit, ssl, url, quality)

# -----------------------------------------------------------------------#


def probe_getinfo(filename):
    """
    Get data stream informations during dragNdrop action.
    It is called by MyListCtrl(wx.ListCtrl) only.
    Return tuple object with two items: (data, None) or (None, error).
    """
    get = wx.GetApp()
    probe = ffprobe(filename, get.appset['ffprobe_cmd'],
                    hide_banner=None, pretty=None)

    return probe
# -------------------------------------------------------------------------#


def volume_detect_process(filelist, time_seq, audiomap):
    """
    Run thread to get audio peak level data and show a
    pop-up dialog with message.
    """
    get = wx.GetApp()
    thread = VolumeDetectThread(time_seq,
                                filelist,
                                audiomap,
                                get.appset['logdir'],
                                get.appset['ffmpeg_cmd']
                                )
    dlgload = PopupDialog(None,
                          _("Videomass - Loading..."),
                          _("Wait....\nAudio peak analysis."))
    dlgload.ShowModal()
    # thread.join()
    data = thread.data
    dlgload.Destroy()

    return data
# -------------------------------------------------------------------------#


def test_conf():
    """
    Call *check_bin.ffmpeg_conf* to get data to test the building
    configurations of the installed or imported FFmpeg executable
    and send it to dialog box.

    """
    get = wx.GetApp()
    out = ff_conf(get.appset['ffmpeg_cmd'], get.appset['ostype'])
    if 'Not found' in out[0]:
        wx.MessageBox(f"\n{out[1]}", "Videomass", wx.ICON_ERROR, None)
        return

    miniframe = ffmpeg_conf.Checkconf(out,
                                      get.appset['ffmpeg_cmd'],
                                      get.appset['ffprobe_cmd'],
                                      get.appset['ffplay_cmd'],
                                      get.appset['ostype'],
                                      )
    miniframe.Show()
# -------------------------------------------------------------------------#


def test_formats():
    """
    Call *check_bin.ff_formats* to get available formats by
    imported FFmpeg executable and send it to dialog box.

    """
    get = wx.GetApp()
    diction = ff_formats(get.appset['ffmpeg_cmd'], get.appset['ostype'])
    if 'Not found' in diction:
        wx.MessageBox(f"\n{diction['Not found']}", "Videomass",
                      wx.ICON_ERROR,
                      None)
        return

    miniframe = ffmpeg_formats.FFmpeg_formats(diction, get.appset['ostype'])
    miniframe.Show()
# -------------------------------------------------------------------------#


def test_codecs(type_opt):
    """
    Call *check_bin.ff_codecs* to get available encoders
    and decoders by FFmpeg executable and send it to
    corresponding dialog box.

    """
    get = wx.GetApp()
    diction = ff_codecs(get.appset['ffmpeg_cmd'],
                        type_opt,
                        get.appset['ostype']
                        )
    if 'Not found' in diction:
        wx.MessageBox(f"\n{diction['Not found']}", "Videomass",
                      wx.ICON_ERROR,
                      None)
        return

    miniframe = ffmpeg_codecs.FFmpeg_Codecs(diction,
                                            get.appset['ostype'],
                                            type_opt
                                            )
    miniframe.Show()
# -------------------------------------------------------------------------#


def findtopic(topic):
    """
    Call * check_bin.ff_topic * to run the ffmpeg command to search
    a certain topic..

    """
    get = wx.GetApp()
    retcod = ff_topics(get.appset['ffmpeg_cmd'], topic, get.appset['ostype'])

    if 'Not found' in retcod[0]:
        notf = (f"\n{retcod[1]}")
        return notf

    return retcod[1]
# -------------------------------------------------------------------------#


def openpath(where):
    """
    Call vdms_threads.opendir.browse to open file browser into
    configuration directory or log directory.

    """
    get = wx.GetApp()
    ret = browse(get.appset['ostype'], where)
    if ret:
        wx.MessageBox(ret, 'Videomass', wx.ICON_ERROR, None)
# -------------------------------------------------------------------------#


def youtubedl_getstatistics(url, ssl):
    """
    Call `YdlExtractInfo` thread to extract data info.
    During this process a wait pop-up dialog is shown.

    Returns a generator.

    Usage example without pop-up dialog:
        thread = YdlExtractInfo(url)
        thread.join()
        data = thread.data
        yield data
    """
    thread = YdlExtractInfo(url, ssl)
    dlgload = PopupDialog(None,
                          _("Videomass - Loading..."),
                          _("Wait....\nRetrieving required data."))
    dlgload.ShowModal()
    # thread.join()
    data = thread.data
    dlgload.Destroy()
    yield data
# --------------------------------------------------------------------------#


def get_github_releases(url, keyname):
    """
    Check for releases data on github page using github API:
        https://developer.github.com/v3/repos/releases/#get-the-latest-release

    see keyname examples here:
    <https://api.github.com/repos/jeanslack/Videomass/releases>

    """
    try:
        response = requests.get(url)
        not_found = None, None

    except Exception as err:
        not_found = 'request error:', err

    else:

        try:
            version = response.json()[f"{keyname}"]

        except Exception as err:
            not_found = 'response error:', err

    if not_found[0]:
        return not_found

    return version, None
# --------------------------------------------------------------------------#


def get_presets(url, dest, msg):
    """
    get latest Videomass presets
    """
    thread = generic_downloads.FileDownloading(url, dest)
    dlgload = PopupDialog(None, _("Videomass - Downloading..."), msg)
    dlgload.ShowModal()
    # thread.join()
    status = thread.data
    dlgload.Destroy()

    return status
# --------------------------------------------------------------------------#


def appimage_update(appimage, script):
    """
    Call appropriate thread to update Python package
    inside Videomass AppImage.
    """
    get = wx.GetApp()  # get data from bootstrap
    logname = 'AppImage_Updates.log'
    logfile = os.path.join(get.appset['logdir'], logname)
    make_log_template(logname, get.appset['logdir'])  # write log file first

    thread = appimage_updater.AppImageUpdate(appimage, script, logfile)

    waitmsg = _('Be patient...\nthis can take a few minutes.')

    dlgload = PopupDialog(None, _("Videomass - Updating..."), waitmsg)
    dlgload.ShowModal()
    # thread.join()
    update = thread.status
    dlgload.Destroy()

    if update:
        return update

    ret = None
    with open(logfile, 'r', encoding='utf8') as fln:
        for line in fln:
            if '**Successfully updated**\n' in line:
                ret = 'success'
    return 'success' if ret == 'success' else 'error'
# --------------------------------------------------------------------------#



# -*- coding: UTF-8 -*-
"""
Name: filedrop.py
Porpose: files drag n drop interface
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2022 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: March.24.2022
Code checker:
    flake8: --ignore F821, W504
    pylint: --ignore E0602, E1101
########################################################

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
import sys
import wx
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_io import io_tools
from videomass.vdms_utils.utils import get_milliseconds
from videomass.vdms_utils.utils import to_bytes


class MyListCtrl(wx.ListCtrl):
    """
    This is the listControl widget. Note that this wideget has DnDPanel
    parent.
    """
    AZURE = '#d9ffff'  # rgb form (wx.Colour(217,255,255))
    RED = '#ea312d'
    YELLOW = '#bd9f00'
    GREENOLIVE = '#6aaf23'
    ORANGE = '#f28924'
    WHITE = '#fbf4f4'  # white for background status bar
    BLACK = '#060505'  # black for background status bar
    # ----------------------------------------------------------------------

    def __init__(self, parent):
        """Constructor"""
        self.index = None
        self.parent = parent  # parent is DnDPanel class
        self.data = self.parent.data
        wx.ListCtrl.__init__(self, parent, style=wx.LC_REPORT)
    # ----------------------------------------------------------------------#

    def dropUpdate(self, path):
        """
        Update list-control during drag and drop

        """
        self.index = self.GetItemCount()
        msg_dir = _("Directories are not allowed, just add files, please.")
        msg_noext = _("File without format extension: please give an "
                      "appropriate extension to the file name, example "
                      "'.mkv', '.avi', '.mp3', etc.")
        msg_badfn = _("Invalid filename. Contains double or single quotes")

        if '"' in path or "'" in path:
            self.parent.statusbar_msg(msg_badfn, MyListCtrl.ORANGE,
                                      MyListCtrl.WHITE)
            return

        if os.path.isdir(path):
            self.parent.statusbar_msg(msg_dir, MyListCtrl.ORANGE,
                                      MyListCtrl.WHITE)
            return

        if os.path.splitext(os.path.basename(path))[1] == '':
            self.parent.statusbar_msg(msg_noext, MyListCtrl.ORANGE,
                                      MyListCtrl.WHITE)
            return

        if not [x for x in self.data if x['format']['filename'] == path]:
            data = io_tools.probe_getinfo(path)

            if data[1]:
                self.parent.statusbar_msg(data[1], MyListCtrl.RED,
                                          MyListCtrl.WHITE)
                return

            data = data[0]
            self.InsertItem(self.index, str(self.index + 1))
            self.SetItem(self.index, 1, path)

            if 'duration' not in data['format'].keys():
                self.SetItem(self.index, 2, 'N/A')
                # NOTE these are my adds in ffprobe data
                data['format']['time'] = '00:00:00.000'
                data['format']['duration'] = 0

            else:
                tdur = data['format']['duration'].split(':')
                sec, msec = tdur[2].split('.')[0], tdur[2].split('.')[1]
                tdur = f'{tdur[0]}h : {tdur[1]}m : {sec} : {msec}'
                self.SetItem(self.index, 2, tdur)
                data.get('format')['time'] = data.get('format').pop('duration')
                time = get_milliseconds(data.get('format')['time'])
                data['format']['duration'] = time

            media = data['streams'][0]['codec_type']
            formatname = data['format']['format_long_name']
            self.SetItem(self.index, 3, f'{media}: {formatname}')
            self.SetItem(self.index, 4, data['format']['size'])
            self.index += 1
            self.data.append(data)
            self.parent.statusbar_msg('', None)
            self.parent.reset_timeline()

        else:
            mess = _("Duplicate files are rejected: > {}").format(path)
            self.parent.statusbar_msg(mess, MyListCtrl.YELLOW,
                                      MyListCtrl.BLACK)
    # ----------------------------------------------------------------------#


class FileDrop(wx.FileDropTarget):
    """
    This is the file drop target
    """
    # ----------------------------------------------------------------------#

    def __init__(self, window):
        """
        Constructor. File Drop targets are subsets of windows
        """
        wx.FileDropTarget.__init__(self)
        self.window = window  # window is MyListCtr class
    # ----------------------------------------------------------------------#

    def OnDropFiles(self, x, y, filenames):
        """
        When files are dropped, write where they were dropped and then
        the file paths themselves
        """
        for filepath in filenames:
            self.window.dropUpdate(filepath)  # update list control

        return True
    # ----------------------------------------------------------------------#


class FileDnD(wx.Panel):
    """
    This panel hosts a listctrl for Drag AND Drop actions on which
    you can add new files, select them, remove them, play them,
    sort them or remove them from the list.

    """
    # CONSTANTS:
    YELLOW = '#bd9f00'
    WHITE = '#fbf4f4'  # white for background status bar
    BLACK = '#060505'  # black for background status bar

    def __init__(self, parent, iconplay):
        """Constructor. This will initiate with an id and a title"""
        get = wx.GetApp()
        appdata = get.appset
        self.parent = parent  # parent is the MainFrame

        if 'wx.svg' in sys.modules:  # available only in wx version 4.1 to up
            bmpplay = get_bmp(iconplay, ((16, 16)))
        else:
            bmpplay = wx.Bitmap(iconplay, wx.BITMAP_TYPE_ANY)
        self.data = self.parent.data_files  # set items list data on parent
        self.file_dest = appdata['outputfile']
        self.selected = None  # tells if an imported file is selected or not
        self.sortingstate = None  # ascending or descending order

        wx.Panel.__init__(self, parent=parent)

        # This builds the list control box:
        self.flCtrl = MyListCtrl(self)  # class MyListCtr
        # Establish the listctrl as a drop target:
        file_drop_target = FileDrop(self.flCtrl)
        self.flCtrl.SetDropTarget(file_drop_target)  # Make drop target.
        # create widgets
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add((0, 10))
        infomsg = _("Drag one or more files below")
        lbl_info = wx.StaticText(self, wx.ID_ANY, label=infomsg)
        sizer.Add(lbl_info, 0, wx.ALL | wx.EXPAND, 5)
        sizer.Add(self.flCtrl, 1, wx.EXPAND | wx.ALL, 2)
        sizer.Add((0, 10))
        sizer_media = wx.BoxSizer(wx.HORIZONTAL)
        self.btn_play = wx.Button(self, wx.ID_ANY, _("Play"))
        self.btn_play.SetBitmap(bmpplay, wx.LEFT)
        self.btn_play.Disable()
        sizer_media.Add(self.btn_play, 1, wx.ALL | wx.EXPAND, 2)
        self.btn_remove = wx.Button(self, wx.ID_REMOVE, "")
        self.btn_remove.Disable()
        sizer_media.Add(self.btn_remove, 1, wx.ALL | wx.EXPAND, 2)
        self.btn_clear = wx.Button(self, wx.ID_CLEAR, "")
        self.btn_clear.Disable()
        sizer_media.Add(self.btn_clear, 1, wx.ALL | wx.EXPAND, 2)
        sizer.Add(sizer_media, 0, wx.EXPAND)
        sizer_outdir = wx.BoxSizer(wx.HORIZONTAL)
        self.btn_save = wx.Button(self, wx.ID_OPEN, "...", size=(35, -1))
        self.text_path_save = wx.TextCtrl(self, wx.ID_ANY, "",
                                          style=wx.TE_PROCESS_ENTER |
                                          wx.TE_READONLY
                                          )
        sizer_outdir.Add(self.text_path_save, 1, wx.ALL | wx.EXPAND, 2)
        sizer_outdir.Add(self.btn_save, 0, wx.ALL |
                         wx.ALIGN_CENTER_HORIZONTAL |
                         wx.ALIGN_CENTER_VERTICAL, 2
                         )
        sizer.Add(sizer_outdir, 0, wx.EXPAND)
        self.SetSizer(sizer)
        # properties
        self.flCtrl.InsertColumn(0, '#', width=30)
        self.flCtrl.InsertColumn(1, _('File Name'), width=350)
        self.flCtrl.InsertColumn(2, _('Duration'), width=230)
        self.flCtrl.InsertColumn(3, _('Media type'), width=200)
        self.flCtrl.InsertColumn(4, _('Size'), width=150)

        if appdata['ostype'] == 'Darwin':
            lbl_info.SetFont(wx.Font(12, wx.DEFAULT, wx.NORMAL, wx.BOLD))
        else:
            lbl_info.SetFont(wx.Font(9, wx.DEFAULT, wx.NORMAL, wx.BOLD))

        self.text_path_save.SetValue(self.file_dest)

        if appdata['outputfile_samedir'] is True:
            self.btn_save.Disable()
            self.text_path_save.Disable()
            self.parent.same_destin = True
            if not appdata['filesuffix'] == '':
                # otherwise must be '' on parent
                self.parent.suffix = appdata['filesuffix']

        # Tooltip
        self.btn_remove.SetToolTip(_('Remove the selected '
                                     'files from the list'))
        self.btn_clear.SetToolTip(_('Delete all files from the list'))
        tip = (_('Set up a temporary folder for conversions'))
        self.btn_save.SetToolTip(tip)
        self.btn_play.SetToolTip(_('Play the selected file in the list'))
        self.text_path_save.SetToolTip(_("Destination folder"))

        # Binding (EVT)
        self.Bind(wx.EVT_BUTTON, self.on_play_select, self.btn_play)
        self.Bind(wx.EVT_BUTTON, self.delete_all, self.btn_clear)
        self.Bind(wx.EVT_BUTTON, self.on_delete_selected, self.btn_remove)
        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.on_select, self.flCtrl)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.on_deselect, self.flCtrl)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.on_double_click,
                  self.flCtrl)
        self.Bind(wx.EVT_LIST_COL_CLICK, self.on_col_click, self.flCtrl)
        self.Bind(wx.EVT_CONTEXT_MENU, self.onContext)
    # ----------------------------------------------------------------------

    def on_col_click(self, event):
        """
        Sort items by LEFT clicking on column headers
        (from ascending to descending and back to ascending).
        For this feature is required to delete all items from
        listctrl and data list before re-loading the same
        items with the new sorted order using `dropUpdate` method.

        if plane to use wx.EVT_LIST_COL_RIGHT_CLICK event:
            `if event.GetEventType() == wx.EVT_LIST_COL_RIGHT_CLICK.typeId:`
                `new.reverse()`
        see: <https://discuss.wxpython.org/t/event-geteventtype/22860/4>
        """
        count = self.flCtrl.GetItemCount()
        curritems = []

        if count > 1:
            if event.GetColumn() in (0, -1):
                return

            for x in range(count):
                curritems.append((self.flCtrl.GetItemText(x, col=1),
                                  self.flCtrl.GetItemText(x, col=2),
                                  self.flCtrl.GetItemText(x, col=3),
                                  self.flCtrl.GetItemText(x, col=4),
                                  ))
            if event.GetColumn() == 1:
                new = sorted(curritems, key=lambda item: item[0])

            elif event.GetColumn() == 2:
                new = sorted(curritems, key=lambda item: item[1])

            elif event.GetColumn() == 3:
                new = sorted(curritems, key=lambda item: item[2])

            elif event.GetColumn() == 4:
                new = sorted(curritems, key=lambda item:
                             to_bytes(''.join(item[3].split()), 'ffmpeg'))

            self.delete_all(self, setstate=False)  # does not setstate here

            if self.sortingstate == 'descending':
                self.sortingstate = 'ascending'

            elif self.sortingstate == 'ascending':
                self.sortingstate = 'descending'

            elif self.sortingstate is None:
                self.sortingstate = 'ascending'

            if self.sortingstate == 'descending':
                new.reverse()
            for f in new:
                self.flCtrl.dropUpdate(f[0])
    # ----------------------------------------------------------------------

    def reset_timeline(self):
        """
        When you drop new files resets the timeline on main_frame.
        it is also needed by the MyListCtrl class.

        """
        self.parent.reset_Timeline()
        if not self.btn_clear.IsEnabled():
            self.btn_clear.Enable()
    # ----------------------------------------------------------------------

    def which(self):
        """
        return topic name by choose_topic.py selection

        """
        return self.parent.topicname
    # ----------------------------------------------------------------------

    def onContext(self, event):
        """
        Create and show a Context Menu
        """
        # only do this part the first time so the events are only bound once
        if not hasattr(self, "popupID1"):
            popupID1 = wx.ID_ANY
            popupID2 = wx.ID_ANY
            self.Bind(wx.EVT_MENU, self.onPopup, id=popupID1)
            self.Bind(wx.EVT_MENU, self.onPopup, id=popupID2)
        # build the menu
        menu = wx.Menu()
        menu.Append(popupID2, _("Play"))
        menu.Append(popupID1, _("Remove"))
        # show the popup menu
        self.PopupMenu(menu)
        menu.Destroy()
    # ----------------------------------------------------------------------

    def onPopup(self, event):
        """
        Evaluate the label string of the menu item selected and starts
        the related process
        """
        itemId = event.GetId()
        menu = event.GetEventObject()
        menuItem = menu.FindItemById(itemId)

        if menuItem.GetItemLabel() == _("Play"):
            self.on_play_select(self)

        elif menuItem.GetItemLabel() == _("Remove"):
            self.on_delete_selected(self)

    # ----------------------------------------------------------------------

    def on_play_select(self, event):
        """
        Playback the selected file

        """
        if not self.selected:
            self.parent.statusbar_msg(_('No file selected'), FileDnD.YELLOW,
                                      FileDnD.BLACK)
        else:
            self.parent.statusbar_msg(_('Add Files'), None)
            index = self.flCtrl.GetFocusedItem()
            item = self.flCtrl.GetItemText(index, 1)
            if self.parent.checktimestamp:
                tstamp = f'-vf "{self.parent.cmdtimestamp}"'
            else:
                tstamp = ""
            io_tools.stream_play(item, self.parent.time_seq,
                                 tstamp, self.parent.autoexit
                                 )
    # ----------------------------------------------------------------------

    def on_delete_selected(self, event):
        """
        Delete a selected file or a bunch of selected files

        """
        if not self.selected:
            self.parent.statusbar_msg(_('No file selected'), FileDnD.YELLOW,
                                      FileDnD.BLACK)
            return

        self.parent.statusbar_msg(_('Add Files'), None)
        item, indexes = -1, []
        while 1:
            item = self.flCtrl.GetNextItem(item,
                                           wx.LIST_NEXT_ALL,
                                           wx.LIST_STATE_SELECTED)
            indexes.append(item)
            if item == -1:
                indexes.remove(-1)
                break

        if self.flCtrl.GetItemCount() == len(indexes):
            self.delete_all(self)
            return

        for num in sorted(indexes, reverse=True):
            self.flCtrl.DeleteItem(num)  # remove selected items
            self.data.pop(num)  # remove selected items from data
        self.reset_timeline()  # delete parent.timeline
        self.on_deselect(self)  # deselect removed file

        for x in range(self.flCtrl.GetItemCount()):
            self.flCtrl.SetItem(x, 0, str(x + 1))  # re-load counter
        return
    # ----------------------------------------------------------------------

    def delete_all(self, event, setstate=True):
        """
        Clear all lines on the listCtrl and delete
        self.data list.
        """
        # self.flCtrl.ClearAll()
        self.flCtrl.DeleteAllItems()
        del self.data[:]
        self.parent.filedropselected = None
        self.reset_timeline()
        self.selected = None
        self.btn_play.Disable()
        self.btn_remove.Disable()
        self.btn_clear.Disable()
        if setstate is True:
            self.sortingstate = None
    # ----------------------------------------------------------------------

    def on_select(self, event):
        """
        Selecting line with mouse or up/down keyboard buttons
        """
        index = self.flCtrl.GetFocusedItem()
        item = self.flCtrl.GetItemText(index, 1)
        self.parent.filedropselected = item
        self.selected = item
        self.btn_play.Enable()
        self.btn_remove.Enable()
    # ----------------------------------------------------------------------

    def on_double_click(self, row):
        """
        Double click or keyboard enter button, open media info
        """
        self.onContext(self)
        # self.parent.ImportInfo(self)
    # ----------------------------------------------------------------------

    def on_deselect(self, event):
        """
        De-selecting a line with mouse by click in empty space of
        the control list
        """
        self.parent.filedropselected = None
        self.selected = None
        self.btn_play.Disable()
        self.btn_remove.Disable()
    # ----------------------------------------------------------------------

    def on_file_save(self, path):
        """
        Set a specific directory for files saving

        """
        self.text_path_save.SetValue("")
        self.text_path_save.AppendText(path)
        self.file_dest = path
    # -----------------------------------------------------------------------

    def statusbar_msg(self, mess, bcolor, fcolor=None):
        """
        Set a status bar message of the parent method.
        bcolor: background, fcolor: foreground
        """
        self.parent.statusbar_msg(f'{mess}', bcolor, fcolor)



# -*- coding: UTF-8 -*-
"""
Name: choose_topic.py
Porpose: shows the topics available in the program
Compatibility: Python3, wxPython Phoenix
Author: Gianluca Pernigotto <jeanlucperni@gmail.com>
Copyright: (c) 2018/2022 Gianluca Pernigotto <jeanlucperni@gmail.com>
license: GPL3
Rev: Apr.06.2022
Code checker: flake8, pylint
########################################################

This file is part of Videomass.

   Videomass is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   Videomass is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Videomass.  If not, see <http://www.gnu.org/licenses/>.
"""
import sys
import wx
from videomass.vdms_utils.get_bmpfromsvg import get_bmp
from videomass.vdms_sys.msg_info import current_release


class Choose_Topic(wx.Panel):
    """
    Helps to choose a topic.
    """
    def __init__(self, parent, OS):
        """
        This is a home panel shown when start Videomass to choose the
        appropriate contextual panel.
        """
        get = wx.GetApp()
        self.appdata = get.appset
        self.icons = parent.icons
        # ----------------------
        self.parent = parent
        self.oS = OS
        version = current_release()

        if 'wx.svg' in sys.modules:  # available only in wx version 4.1 to up
            bmpAVconv = get_bmp(self.icons['A/V-Conv'], ((48, 48)))
            bmpPrstmng = get_bmp(self.icons['presets_manager'], ((48, 48)))
            bmpYdl = get_bmp(self.icons['youtube'], ((48, 48)))
            bmpConcat = get_bmp(self.icons['concatenate'], ((48, 48)))
            bmpSlideshow = get_bmp(self.icons['slideshow'], ((48, 48)))
            bmpTopictures = get_bmp(self.icons['videotopictures'], ((48, 48)))
        else:
            bmpAVconv = wx.Bitmap(self.icons['A/V-Conv'], wx.BITMAP_TYPE_ANY)
            bmpPrstmng = wx.Bitmap(self.icons['presets_manager'],
                                   wx.BITMAP_TYPE_ANY)
            bmpYdl = wx.Bitmap(self.icons['youtube'], wx.BITMAP_TYPE_ANY)
            bmpConcat = wx.Bitmap(self.icons['concatenate'],
                                  wx.BITMAP_TYPE_ANY)
            bmpSlideshow = wx.Bitmap(self.icons['slideshow'],
                                     wx.BITMAP_TYPE_ANY)
            bmpTopictures = wx.Bitmap(self.icons['videotopictures'],
                                      wx.BITMAP_TYPE_ANY)

        wx.Panel.__init__(self, parent, -1, style=wx.TAB_TRAVERSAL)

        welcome = wx.StaticText(self, wx.ID_ANY, (_("Welcome to Videomass")))
        version = wx.StaticText(self, wx.ID_ANY, (_('Version {}'
                                                    ).format(version[2])))
        sizer_base = wx.BoxSizer(wx.VERTICAL)

        sizer_base.Add(120, 120)
        sizer_base.Add(welcome, 0, wx.ALIGN_CENTER_VERTICAL |
                       wx.ALIGN_CENTER_HORIZONTAL, 0
                       )
        sizer_base.Add(20, 20)
        sizer_base.Add(version, 0, wx.BOTTOM |
                       wx.ALIGN_CENTER_VERTICAL |
                       wx.ALIGN_CENTER_HORIZONTAL, 0
                       )

        if self.oS == 'Windows':
            style = wx.BU_LEFT | wx.BORDER_NONE
        else:
            style = wx.BU_LEFT

        self.presets_mng = wx.Button(self, wx.ID_ANY, _('Presets Manager'),
                                     size=(300, -1), style=style)
        self.presets_mng.SetBitmap(bmpPrstmng, wx.LEFT)

        self.avconv = wx.Button(self, wx.ID_ANY, _('AV-Conversions'),
                                size=(300, -1), style=style)
        self.avconv.SetBitmap(bmpAVconv, wx.LEFT)

        self.conc = wx.Button(self, wx.ID_ANY, _('Concatenate media files'),
                              size=(300, -1), style=style)
        self.conc.SetBitmap(bmpConcat, wx.LEFT)

        self.youtube = wx.Button(self, wx.ID_ANY, _('YouTube Downloader'),
                                 size=(300, -1), style=style)
        self.youtube.SetBitmap(bmpYdl, wx.LEFT)

        self.slideshow = wx.Button(self, wx.ID_ANY,
                                   _('Still Image Maker'),
                                   size=(300, -1), style=style)
        self.slideshow.SetBitmap(bmpSlideshow, wx.LEFT)

        self.videotoimages = wx.Button(self, wx.ID_ANY,
                                       _('From Movie to Pictures'),
                                       size=(300, -1), style=style)
        self.videotoimages.SetBitmap(bmpTopictures, wx.LEFT)

        grid_buttons = wx.FlexGridSizer(3, 2, 20, 20)
        grid_buttons.AddMany([(self.presets_mng, 0, wx.EXPAND, 5),
                              (self.avconv, 0, wx.EXPAND, 5),
                              (self.conc, 0, wx.EXPAND, 5),
                              (self.youtube, 0, wx.EXPAND, 5),
                              (self.slideshow, 0, wx.EXPAND, 5),
                              (self.videotoimages, 0, wx.EXPAND, 5),
                              ])
        sizer_base.Add(80, 80)
        sizer_base.Add(grid_buttons, 1, wx.ALIGN_CENTER_VERTICAL |
                       wx.ALIGN_CENTER_HORIZONTAL, 5
                       )
        self.SetSizerAndFit(sizer_base)

        # ---------------------- Tooltips
        tip = (_('Create, edit and use quickly your favorite FFmpeg '
                 'presets and profiles with full formats support and '
                 'codecs.'))
        self.presets_mng.SetToolTip(tip)
        tip = (_('A set of useful tools for audio and video conversions. '
                 'Save your profiles and reuse them with Presets '
                 'Manager.'))
        self.avconv.SetToolTip(tip)
        tip = (_('Easily download videos and audio in different '
                 'formats and quality from YouTube, Facebook and '
                 'more sites.'))
        self.youtube.SetToolTip(tip)
        tip = (_('Concatenate multiple media files based on import '
                 'order without re-encoding'))
        self.conc.SetToolTip(tip)
        tip = (_('Create video from a sequence of images, based on import '
                 'order, with the ability to add an audio file.'))
        self.slideshow.SetToolTip(tip)
        tip = (_('Extract images (frames) from your movies '
                 'in JPG, PNG, BMP, GIF formats.'))
        self.videotoimages.SetToolTip(tip)

        if self.oS == 'Darwin':
            welcome.SetFont(wx.Font(14, wx.SWISS, wx.NORMAL, wx.BOLD))
            version.SetFont(wx.Font(11, wx.SWISS, wx.NORMAL, wx.NORMAL))
        else:
            welcome.SetFont(wx.Font(12, wx.SWISS, wx.NORMAL, wx.NORMAL))
            version.SetFont(wx.Font(9, wx.SWISS, wx.NORMAL, wx.LIGHT))

        if hasattr(wx.SystemSettings, 'GetAppearance'):
            appear = wx.SystemSettings.GetAppearance()
            if appear.IsDark():
                self.SetBackgroundColour('#070b0e')  # blue darkness
                welcome.SetForegroundColour('#777072')  # grey
                version.SetForegroundColour('#777072')  # grey
            else:
                self.SetBackgroundColour('#f2efe6')  # beige
                welcome.SetForegroundColour('#171b12')  # black
                version.SetForegroundColour('#171b12')  # black

        self.SetMinSize(tuple(self.parent.appdata['panel_size']))

        self.Bind(wx.EVT_BUTTON, self.on_Conv, self.avconv)
        self.Bind(wx.EVT_BUTTON, self.on_Prst_mng, self.presets_mng)
        self.Bind(wx.EVT_BUTTON, self.on_Conc, self.conc)
        self.Bind(wx.EVT_BUTTON, self.on_slideshow, self.slideshow)
        self.Bind(wx.EVT_BUTTON, self.on_to_pictures, self.videotoimages)
        self.Bind(wx.EVT_BUTTON, self.on_YoutubeDL, self.youtube)
    # ------------------------------------------------------------------#

    def on_Prst_mng(self, event):
        """
        Open drag N drop interface to switch on Presets Manager panel
        """
        self.parent.switch_file_import(self, 'Presets Manager')
    # ------------------------------------------------------------------#

    def on_Conv(self, event):
        """
        Open drag N drop interface to switch on AVconversions panel
        """
        self.parent.switch_file_import(self, 'Audio/Video Conversions')
    # ------------------------------------------------------------------#

    def on_Conc(self, event):
        """
        Open drag N drop interface to switch on Concatenate panel
        """
        self.parent.switch_file_import(self, 'Concatenate Demuxer')
    # ------------------------------------------------------------------#

    def on_slideshow(self, event):
        """
        Open drag N drop interface to switch on Image Sequence to Video panel
        """
        self.parent.switch_file_import(self, 'Image Sequence to Video')
    # ------------------------------------------------------------------#

    def on_to_pictures(self, event):
        """
        Open drag N drop interface to switch on Video to Pictures panel
        """
        self.parent.switch_file_import(self, 'Video to Pictures')
    # ------------------------------------------------------------------#

    def on_YoutubeDL(self, event):
        """
        Check the existence of youtube-dl based on the set attributes
        of the bootstrap, then open the text interface, otherwise it
        requires installation.

        """
        if self.appdata['downloader'] == 'disabled':
            wx.MessageBox(_("The downloader is disabled. "
                            "Check your preferences."),
                          "Videomass", wx.ICON_INFORMATION, self)
            return True

        # PYLIBYDL: None if used else 'string error'
        if self.appdata['PYLIBYDL'] is None:
            self.parent.switch_text_import(self, 'Youtube Downloader')
            return None

        wx.MessageBox(_("ERROR: {0}\n\n{1} is not installed, use your "
                        "package manager to install it.").format(
                      self.appdata['PYLIBYDL'], self.appdata['downloader']),
                      "Videomass", wx.ICON_ERROR, self)
        return True
